package net.maunium.bukkit.MauBans;

import java.io.Serializable;
import java.util.Arrays;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;

public class Ban implements Serializable {
	private static final long serialVersionUID = 29301000;
	public static MauBans plugin;
	private String reason, bannedBy;
	private UUID player;
	private long timeout;
	
	public Ban(UUID p, String reason, String bannedBy, long timeout) {
		this.player = p;
		this.reason = reason;
		this.bannedBy = bannedBy;
		this.timeout = timeout;
	}
	
	public UUID getPlayer() {
		return player;
	}
	
	public String getBannedBy() {
		return bannedBy;
	}
	
	public String getReason() {
		return reason;
	}
	
	public String getReadableReason() {
		if (isPermanent()) return plugin.format("tempbanned", bannedBy, reason);
		else return plugin.format("banned", bannedBy, reason) + "\n" + plugin.format("timeout", getDurationBreakdown(getTimeToTimeout(), 0));
	}
	
	public long getTimeoutAt() {
		if (isPermanent()) return -1;
		else return timeout;
	}
	
	public long getTimeToTimeout() {
		if (isPermanent()) return -1;
		else return timeout - System.currentTimeMillis();
	}
	
	public boolean isTimedOut() {
		if (!isPermanent() && getTimeoutAt() <= System.currentTimeMillis()) return true;
		else return false;
	}
	
	public boolean isPermanent() {
		return timeout < 0;
	}
	
	public static Ban parseBan(String s) {
		try {
			String[] temp = s.split(Pattern.quote("|"));
			if (temp.length == 4) {
				UUID p;
				long timeout;
				try {
					p = UUID.fromString(temp[0]);
				} catch (Exception e) {
					throw new IllegalArgumentException("Invalid argument (parse error): " + temp[0]);
				}
				try {
					timeout = Long.parseLong(temp[3]);
				} catch (Exception e) {
					throw new IllegalArgumentException("Invalid argument (parse error): " + temp[3]);
				}
				return new Ban(p, temp[1], temp[2], timeout);
			} else throw new IllegalArgumentException("Invalid argument (amount of entries): " + Arrays.toString(temp) + ", " + temp.length);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	@Override
	public String toString() {
		return player + "|" + reason + "|" + bannedBy + "|" + timeout;
	}
	
	public static String toString(Ban b) {
		return b.player + "|" + b.reason + "|" + b.timeout;
	}
	
	public static String getDurationBreakdown(long millis, int mode) {
		if (millis < 0) return "never";
		long days = TimeUnit.MILLISECONDS.toDays(millis);
		millis -= TimeUnit.DAYS.toMillis(days);
		long hours = TimeUnit.MILLISECONDS.toHours(millis);
		millis -= TimeUnit.HOURS.toMillis(hours);
		long minutes = TimeUnit.MILLISECONDS.toMinutes(millis);
		millis -= TimeUnit.MINUTES.toMillis(minutes);
		long seconds = TimeUnit.MILLISECONDS.toSeconds(millis);
		
		String md = mode == 0 ? "in" : "for";
		
		StringBuilder sb = new StringBuilder(48);
		if (days > 0) {
			sb.append(days);
			sb.append(" ");
			sb.append(plugin.format("date." + md + ".days"));
			sb.append(" ");
		}
		if (hours > 0) {
			sb.append(hours);
			sb.append(" ");
			sb.append(plugin.format("date." + md + ".hours"));
			sb.append(" ");
		}
		if (minutes > 0) {
			sb.append(minutes);
			sb.append(" ");
			sb.append(plugin.format("date." + md + ".minutes"));
			sb.append(" ");
		}
		if (seconds > 0) {
			sb.append(seconds);
			sb.append(" ");
			sb.append(plugin.format("date." + md + ".seconds"));
			sb.append(" ");
		}
		sb.deleteCharAt(sb.length() - 1);
		if (sb.length() == 0) sb.append(plugin.format("date." + md + ".none"));
		return sb.toString();
	}
}
