package net.maunium.bukkit.MauBans;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.UUID;

import org.bukkit.BanList;
import org.bukkit.ChatColor;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;

import net.maunium.bukkit.MauBans.CommandExecutors.CommandBan;
import net.maunium.bukkit.MauBans.CommandExecutors.CommandBanIP;
import net.maunium.bukkit.MauBans.CommandExecutors.CommandKick;
import net.maunium.bukkit.MauBans.CommandExecutors.CommandSeen;
import net.maunium.bukkit.MauBans.CommandExecutors.CommandTempban;
import net.maunium.bukkit.MauBans.CommandExecutors.CommandUnban;
import net.maunium.bukkit.MauBans.CommandExecutors.CommandUnbanIP;
import net.maunium.bukkit.MauBans.CommandExecutors.CommandWarn;
import net.maunium.bukkit.MauBukLib.I18n;

public class MauBans extends JavaPlugin {
	public final String name = "MauBans", author = "Tulir293", stag = ChatColor.DARK_GREEN + "[" + ChatColor.GREEN + name + ChatColor.DARK_GREEN + "] " + ChatColor.GRAY, errtag = ChatColor.DARK_RED
			+ "[" + ChatColor.RED + name + ChatColor.DARK_RED + "] " + ChatColor.RED;
	
	private Map<UUID, Ban> bans = new HashMap<UUID, Ban>();
	private Map<String, String> ipbans = new HashMap<String, String>();
	private Map<UUID, Mute> mutes = new HashMap<UUID, Mute>();
	
	private I18n i18n;
	public MauVanish v;
	
	@Override
	public void onEnable() {
		long st = System.currentTimeMillis();
		this.saveDefaultConfig();
		
		Ban.plugin = this;
		this.saveResource("en_US.lang", true);
		this.saveResource("fi_FI.lang", true);
		try {
			i18n = I18n.createInstance(getDataFolder(), getConfig().getString("language"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		for (String s : getConfig().getStringList("banned-players")) {
			Ban b = Ban.parseBan(s);
			bans.put(b.getPlayer(), b);
		}
		
		if (getConfig().contains("ip-bans")) for (Entry<String, Object> s : getConfig().getConfigurationSection("ip-bans").getValues(false).entrySet())
			ipbans.put(s.getKey().replace("-", "."), s.getValue().toString());
		
		this.getCommand("ban").setExecutor(new CommandBan(this));
		this.getCommand("banip").setExecutor(new CommandBanIP(this));
		this.getCommand("warn").setExecutor(new CommandWarn(this));
		this.getCommand("kick").setExecutor(new CommandKick(this));
		this.getCommand("tempban").setExecutor(new CommandTempban(this));
		this.getCommand("unban").setExecutor(new CommandUnban(this));
		this.getCommand("unbanip").setExecutor(new CommandUnbanIP(this));
		this.getCommand("seen").setExecutor(new CommandSeen(this));
		this.getServer().getPluginManager().registerEvents(new PlayerLoginListener(this), this);
		this.getServer().getPluginManager().registerEvents(new ChatListener(this), this);
		this.getServer().getPluginManager().registerEvents(v = new MauVanish(this), this);
		
		long et = System.currentTimeMillis();
		int lt = (int) (et - st);
		getLogger().info(name + " by " + author + " enabled in " + lt + "ms.");
	}
	
	@Override
	public void onDisable() {
		long st = System.currentTimeMillis();
		
		List<String> temp = new ArrayList<String>();
		for (Entry<UUID, Ban> e : bans.entrySet())
			temp.add(e.getValue().toString());
		
		getConfig().set("banned-players", temp);
		
		getConfig().createSection("ip-bans");
		for (Entry<String, String> e : ipbans.entrySet())
			getConfig().set("ip-bans." + e.getKey().replace(".", "-"), e.getValue());
		
		this.saveConfig();
		
		long et = System.currentTimeMillis();
		int lt = (int) (et - st);
		getLogger().info(name + " by " + author + " disabled in " + lt + "ms.");
	}
	
	public boolean isMuted(UUID uuid) {
		if (mutes.containsKey(uuid)) {
			if (mutes.get(uuid).isTimedOut()) {
				getLogger().info("Unmuting " + uuid + ". Reason: Mute timed out.");
				mutes.remove(uuid);
				return false;
			} else return true;
		} else return false;
	}
	
	public boolean isBanned(UUID uuid) {
		if (bans.containsKey(uuid)) {
			if (bans.get(uuid).isTimedOut()) {
				getLogger().info("Unbanning " + uuid + ". Reason: Ban timed out.");
				bans.remove(uuid);
				return false;
			} else return true;
		} else return false;
	}
	
	public boolean banip(String ip, String reason) {
		if (!ipbans.containsKey(ip)) {
			ipbans.put(ip, reason);
			getServer().banIP(ip);
			return true;
		} else return false;
	}
	
	public boolean unbanip(String ip) {
		if (isIpBanned(ip)) {
			ipbans.remove(ip);
			getServer().unbanIP(ip);
			return true;
		} else return false;
	}
	
	public boolean isIpBanned(String ip) {
		return ipbans.containsKey(ip);
	}
	
	public String getIpBan(String ip) {
		if (isIpBanned(ip)) return ipbans.get(ip);
		else return null;
	}
	
	public Ban getBan(UUID uuid) {
		if (isBanned(uuid)) return bans.get(uuid);
		else return null;
	}
	
	public Mute getMute(UUID uuid) {
		if (isMuted(uuid)) return mutes.get(uuid);
		else return null;
	}
	
	public Ban ban(UUID uuid, String name, String bannedBy, String reason) {
		Ban b = new Ban(uuid, reason, bannedBy, -1);
		bans.put(uuid, b);
		getServer().getBanList(BanList.Type.NAME).addBan(name, reason, null, bannedBy);
		Player p = getServer().getPlayer(uuid);
		if (p != null) p.kickPlayer(b.getReadableReason());
		return b;
	}
	
	public Ban tempban(UUID uuid, String name, String bannedBy, String reason, long timeout) {
		Ban b = new Ban(uuid, reason, bannedBy, timeout);
		bans.put(uuid, b);
		getServer().getBanList(BanList.Type.NAME).addBan(name, reason, new Date(timeout), bannedBy);
		Player p = getServer().getPlayer(uuid);
		if (p != null) p.kickPlayer(b.getReadableReason());
		return b;
	}
	
	public void mute(UUID uuid, String mutedBy, String reason, long timeout) {
		mutes.put(uuid, new Mute(uuid, mutedBy, reason, timeout));
	}
	
	public boolean unban(UUID uuid, String name) {
		if (isBanned(uuid)) {
			bans.remove(uuid);
			getServer().getBanList(BanList.Type.NAME).pardon(name);
			return true;
		} else return false;
	}
	
	public String format(String s, Object... args) {
		return i18n.format(s, args);
	}
}
