package net.maunium.bukkit.MauBukLib;

import org.bukkit.ChatColor;

public class ChatFormatter {
	/**
	 * Turn all color, format and magic codes into actual formatting ({@link ChatColor})
	 * 
	 * @param repl The string to format
	 * @return A formatted string
	 */
	public static String full(String repl) {
		repl = color(repl, false);
		repl = format(repl, false);
		repl = magic(repl, false);
		repl = repl.replace("&" + ChatColor.COLOR_CHAR, "&");
		repl = repl.replace("&&", "&");
		return repl;
	}
	
	/**
	 * Reverses all color, formatting and magic.
	 * 
	 * @param repl The formatted string
	 * @return The string without real formatting
	 */
	public static String reverse(String repl, boolean f) {
		repl.replace("&", "&&");
		repl = repl.replace(ChatColor.BLACK.toString(), "&0");
		repl = repl.replace(ChatColor.DARK_BLUE.toString(), "&1");
		repl = repl.replace(ChatColor.DARK_GREEN.toString(), "&2");
		repl = repl.replace(ChatColor.DARK_AQUA.toString(), "&3");
		repl = repl.replace(ChatColor.DARK_RED.toString(), "&4");
		repl = repl.replace(ChatColor.DARK_PURPLE.toString(), "&5");
		repl = repl.replace(ChatColor.GOLD.toString(), "&6");
		repl = repl.replace(ChatColor.GRAY.toString(), "&7");
		repl = repl.replace(ChatColor.DARK_GRAY.toString(), "&8");
		repl = repl.replace(ChatColor.BLUE.toString(), "&9");
		repl = repl.replace(ChatColor.GREEN.toString(), "&a");
		repl = repl.replace(ChatColor.AQUA.toString(), "&b");
		repl = repl.replace(ChatColor.RED.toString(), "&c");
		repl = repl.replace(ChatColor.LIGHT_PURPLE.toString(), "&d");
		repl = repl.replace(ChatColor.YELLOW.toString(), "&e");
		repl = repl.replace(ChatColor.WHITE.toString(), "&f");
		repl = repl.replace(ChatColor.BOLD.toString(), "&l");
		repl = repl.replace(ChatColor.STRIKETHROUGH.toString(), "&m");
		repl = repl.replace(ChatColor.UNDERLINE.toString(), "&n");
		repl = repl.replace(ChatColor.ITALIC.toString(), "&o");
		repl = repl.replace(ChatColor.MAGIC.toString(), "&k");
		repl = repl.replace(ChatColor.RESET.toString(), "&r");
		return repl;
	}
	
	/**
	 * Turn color codes (& + code) into actual colors ({@link ChatColor})
	 * 
	 * @param repl The string to format
	 * @return The string with formatting
	 */
	public static String color(String repl) {
		return color(repl, true);
	}
	
	/**
	 * Turn format codes (& + code) into actual formatting ({@link ChatColor})
	 * 
	 * @param repl The string to format
	 * @return The string with formatting
	 */
	public static String format(String repl) {
		return format(repl, true);
	}
	
	/**
	 * Turn magic codes (&k) into actual magic ({@link ChatColor#MAGIC})
	 * 
	 * @param repl The string to format
	 * @return The string with formatting
	 */
	public static String magic(String repl) {
		return magic(repl, true);
	}
	
	/**
	 * Turn color codes (& + code) into actual colors ({@link ChatColor})
	 * 
	 * @param repl The string to format
	 * @param f Whether or not to cancel formatting if there are two &'s
	 * @return The string with formatting
	 */
	public static String color(String repl, boolean f) {
		repl = repl.replace("&0", ChatColor.BLACK.toString());
		repl = repl.replace("&1", ChatColor.DARK_BLUE.toString());
		repl = repl.replace("&2", ChatColor.DARK_GREEN.toString());
		repl = repl.replace("&3", ChatColor.DARK_AQUA.toString());
		repl = repl.replace("&4", ChatColor.DARK_RED.toString());
		repl = repl.replace("&5", ChatColor.DARK_PURPLE.toString());
		repl = repl.replace("&6", ChatColor.GOLD.toString());
		repl = repl.replace("&7", ChatColor.GRAY.toString());
		repl = repl.replace("&8", ChatColor.DARK_GRAY.toString());
		repl = repl.replace("&9", ChatColor.BLUE.toString());
		repl = repl.replace("&a", ChatColor.GREEN.toString());
		repl = repl.replace("&b", ChatColor.AQUA.toString());
		repl = repl.replace("&c", ChatColor.RED.toString());
		repl = repl.replace("&d", ChatColor.LIGHT_PURPLE.toString());
		repl = repl.replace("&e", ChatColor.YELLOW.toString());
		repl = repl.replace("&f", ChatColor.WHITE.toString());
		repl = repl.replace("&r", ChatColor.RESET.toString());
		if (f){
			repl = repl.replace("&" + ChatColor.COLOR_CHAR, "&");
			repl = repl.replace("&&", "&");
		}
		return repl;
	}
	
	/**
	 * Turn format codes (& + code) into actual formatting ({@link ChatColor})
	 * 
	 * @param repl The string to format
	 * @param f Whether or not to cancel formatting if there are two &'s
	 * @return The formatted string
	 */
	public static String format(String repl, boolean f) {
		repl = repl.replace("&l", ChatColor.BOLD.toString());
		repl = repl.replace("&m", ChatColor.STRIKETHROUGH.toString());
		repl = repl.replace("&n", ChatColor.UNDERLINE.toString());
		repl = repl.replace("&o", ChatColor.ITALIC.toString());
		repl = repl.replace("&r", ChatColor.RESET.toString());
		if (f){
			repl = repl.replace("&" + ChatColor.COLOR_CHAR, "&");
			repl = repl.replace("&&", "&");
		}
		return repl;
	}
	
	/**
	 * Turn magic codes (&k) into actual magic ({@link ChatColor#MAGIC})
	 * 
	 * @param repl The string to format
	 * @param f Whether or not to cancel formatting if there are two &'s
	 * @return The formatted string
	 */
	public static String magic(String repl, boolean f) {
		repl = repl.replace("&k", ChatColor.MAGIC.toString());
		repl = repl.replace("&r", ChatColor.RESET.toString());
		if (f){
			repl = repl.replace("&" + ChatColor.COLOR_CHAR, "&");
			repl = repl.replace("&&", "&");
		}
		return repl;
	}
}
