package net.maunium.bukkit.MauLand;

import java.util.UUID;

import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.WeatherType;
import org.bukkit.World;
import org.bukkit.World.Environment;
import org.bukkit.WorldCreator;
import org.bukkit.WorldType;
import org.bukkit.block.Block;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.metadata.FixedMetadataValue;
import org.bukkit.potion.PotionEffect;

public class WorldManager {
	private MauLand plugin;
	private UUID worldUUID;
	
	public WorldManager(MauLand plugin) {
		this.plugin = plugin;
	}
	
	public void load() {
		WorldCreator wc = new WorldCreator("mauland");
		wc.environment(Environment.NORMAL);
		wc.generateStructures(true);
		wc.type(WorldType.NORMAL);
		wc.seed(-6875351228615670683L);
		World w = wc.createWorld();
		w.setSpawnLocation(0, 100, 0);
		w.setKeepSpawnInMemory(false);
		w.setSpawnFlags(false, true);
		this.worldUUID = w.getUID();
	}
	
	public World getWorld(){
		return plugin.getServer().getWorld(worldUUID);
	}
	
	public Location getSpawn(){
		return new Location(getWorld(), -4.5, 114, -1.5, -45, 25);
	}
	
	public void save() {
		getWorld().save();
	}
	
	public void close() {
		plugin.getServer().unloadWorld(getWorld(), true);
	}
	
	public boolean isMauLand(World w) {
		if (w.getUID().equals(worldUUID)) return true;
		else return false;
	}
	
	public boolean isMauLand(Entity e) {
		if (e.getLocation().getWorld().getUID().equals(worldUUID)) return true;
		else return false;
	}
	
	public boolean isMauLand(Block b) {
		if (b.getLocation().getWorld().getUID().equals(worldUUID)) return true;
		else return false;
	}
	
	public boolean isMauLand(Location l) {
		if (l.getWorld().getUID().equals(worldUUID)) return true;
		else return false;
	}
	
	public boolean isMauLand(UUID uuid) {
		if (uuid.equals(worldUUID)) return true;
		else return false;
	}
	
	public void teleportOut(Player p) {
		if (!p.hasPermission("mauland.trusted")) {
			p.setGameMode(GameMode.SURVIVAL);
			p.getInventory().setArmorContents(new ItemStack[p.getInventory().getArmorContents().length]);
			p.getInventory().setContents(new ItemStack[p.getInventory().getContents().length]);
			p.setAllowFlight(false);
			p.setFlying(false);
			p.setLevel(0);
			p.setExp(0);
			for (PotionEffect pet : p.getActivePotionEffects())
				p.removePotionEffect(pet.getType());
		}
		p.resetPlayerWeather();
		p.resetPlayerTime();
		p.setMetadata(plugin.tpMeta, new FixedMetadataValue(plugin, true));
		p.teleport(new Location(plugin.getServer().getWorld("world"), -15.5, 83, 144.5, 0, 0));
		p.sendMessage(plugin.stag + "You have left MauLand.");
	}
	
	public void teleportIn(Player p) {
		int mode = -1;
		if (p.hasPermission("mauland.access")) {
			if (isEmpty(p.getInventory().getArmorContents()) && isEmpty(p.getInventory().getContents()) || p.hasPermission("mauland.trusted")) {
				p.setMetadata(plugin.tpMeta, new FixedMetadataValue(plugin, true));
				if (p.teleport(getSpawn())) {
					p.setGameMode(GameMode.CREATIVE);
					mode = 1;
					p.sendMessage(plugin.stag + "Welcome to MauLand. You are in creative mode and you can build.");
				} else p.removeMetadata(plugin.tpMeta, plugin);
			} else p.sendMessage(plugin.errtag + "Your inventory is not empty. Please empty your inventory before entering MauLand.");
		} else if (p.hasPermission("mauland.spectate")) {
			if (isEmpty(p.getInventory().getArmorContents()) && isEmpty(p.getInventory().getContents())) {
				p.setMetadata(plugin.tpMeta, new FixedMetadataValue(plugin, true));
				if (p.teleport(getSpawn())) {
					p.setAllowFlight(true);
					mode = 0;
					p.sendMessage(plugin.stag + "Welcome to MauLand. You are in spectator mode and thus you can't build.");
				} else p.removeMetadata(plugin.tpMeta, plugin);
			} else p.sendMessage(plugin.errtag + "Your inventory is not empty. Please empty your inventory before entering MauLand.");
		} else {
			p.sendMessage(plugin.errtag + "You do not have the permissions to enter MauLand.");
			return;
		}
		if (mode > -1) {
			p.setPlayerTime(6000, false);
			for (PotionEffect pet : p.getActivePotionEffects())
				p.removePotionEffect(pet.getType());
			p.setPlayerWeather(WeatherType.CLEAR);
		}
	}
	
	private boolean isEmpty(ItemStack... array) {
		for (ItemStack ts : array)
			if (ts != null && !ts.getType().equals(Material.AIR)) return false;
		return true;
	}
}
