package net.maunium.bukkit.MauLaunchers;

import java.io.File;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import org.bukkit.ChatColor;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.configuration.serialization.ConfigurationSerialization;
import org.bukkit.plugin.java.JavaPlugin;

public class MauLaunchers extends JavaPlugin {
	
	public String version;
	public final String name = "MauLaunchers", author = "Tulir293", stag = ChatColor.DARK_GREEN + "[" + ChatColor.GREEN + name + ChatColor.DARK_GREEN + "] " + ChatColor.GRAY,
			errtag = ChatColor.DARK_RED + "[" + ChatColor.RED + name + ChatColor.DARK_RED + "] " + ChatColor.RED, fallMeta = "MauLaunchers-Disable_fall_damage";
	private Map<Block, Launcher> launchers = new HashMap<Block, Launcher>();
	
	@Override
	public void onEnable() {
		long st = System.currentTimeMillis();
		version = this.getDescription().getVersion();
		this.saveDefaultConfig();
		
		Launcher.setHost(this);
		ConfigurationSerialization.registerClass(Launcher.class);
		
		for (Object o : getConfig().getList("launchers")) {
			if (o instanceof Launcher) {
				Launcher l = (Launcher) o;
				launchers.put(l.getBlock(), l);
				l.activate();
			}
		}
		
		importOldConfig();
		
		this.getServer().getPluginManager().registerEvents(new PlayerMoveListener(), this);
		this.getServer().getPluginManager().registerEvents(new PlayerInteractListener(this), this);
		this.getServer().getPluginManager().registerEvents(new PlayerFallListener(this), this);
		this.getCommand("maulaunchers").setExecutor(new CommandLaunchers(this));
		
		int et = (int) (System.currentTimeMillis() - st);
		getLogger().info(name + " v" + version + " by " + author + " enabled in " + et + "ms.");
	}
	
	@Override
	public void onDisable() {
		long st = System.currentTimeMillis();
		
		getConfig().set("launchers", Arrays.asList(launchers.values().toArray(new Launcher[0])));
		this.saveConfig();
		
		int et = (int) (System.currentTimeMillis() - st);
		getLogger().info(name + " v" + version + " by " + author + " disabled in " + et + "ms.");
	}
	
	public void importOldConfig() {
		File f = new File(this.getDataFolder(), "playerlauncher.yml");
		if (f.exists()) {
			YamlConfiguration old = YamlConfiguration.loadConfiguration(f);
			int confVer = old.getInt("config-version", 0);
			if (confVer == 0) {
				getLogger().info("Found playerlauncher.yml with config version 0.");
				for (int i = 1; i < old.getInt("amount-of-launchers") + 1; i++) {
					World w = getServer().getWorld(old.getString("launchers." + i + ".world"));
					Block b = w.getBlockAt(old.getInt("launchers." + i + ".block-x"), old.getInt("launchers." + i + ".block-y"), old.getInt("launchers." + i + ".block-z"));
					double x = old.getDouble("launchers." + i + ".launcher-x");
					double y = old.getDouble("launchers." + i + ".launcher-y");
					double z = old.getDouble("launchers." + i + ".launcher-z");
					Launcher l = new Launcher(b, x, y, z);
					addLauncher(l);
					getLogger().fine("Imported " + l);
				}
				getLogger().info("Imported all PlayerLauncher launchers!");
			} else if (confVer == 1) {
				getLogger().info("Found playerlauncher.yml with config version 1.");
				for (int i = 1; i < old.getInt("amount-of-launchers") + 1; i++) {
					String world = old.getString("launchers." + i + ".world");
					World w = null;
					if (world != null) {
						if (getServer().getWorld(world) != null) w = getServer().getWorld(world);
						else continue;
					} else continue;
					String blockLoc = old.getString("launchers." + i + ".block-loc");
					String[] blTemp = blockLoc.split(", ");
					Block b = null;
					
					try {
						b = w.getBlockAt(Integer.parseInt(blTemp[0]), Integer.parseInt(blTemp[1]), Integer.parseInt(blTemp[2]));
					} catch (NumberFormatException e) {
						continue;
					}
					
					String vector = old.getString("launchers." + i + ".vector");
					String[] vTemp = vector.split(", ");
					double x, y, z;
					try {
						x = Double.parseDouble(vTemp[0]);
						y = Double.parseDouble(vTemp[1]);
						z = Double.parseDouble(vTemp[2]);
					} catch (NumberFormatException e) {
						continue;
					}
					Launcher l = new Launcher(b, x, y, z);
					addLauncher(l);
					getLogger().fine("Imported " + l);
				}
				getLogger().info("Imported all PlayerLauncher launchers!");
			} else getLogger().warning("Found playerlauncher.yml with unknown config version");
			f.renameTo(new File(this.getDataFolder(), "playerlauncher.yml.imported"));
		}
	}
	
	public boolean addLauncher(Launcher l) {
		if (launchers.containsKey(l.getBlock())) launchers.remove(l.getBlock());
		launchers.put(l.getBlock(), l);
		return l.activate();
	}
	
	public boolean removeLauncher(Block b) {
		if (launchers.containsKey(b)) {
			launchers.get(b).deactivate();
			launchers.remove(b);
		} else if (Launcher.isLauncher(b)) Launcher.deactivate(b);
		else return false;
		return true;
	}
	
	public Launcher getLauncher(Block b) {
		return launchers.get(b);
	}
}