package net.maunium.bukkit.MauSongPlayer;


public abstract class MauPlayer {
	private MauSong song;
	
	private boolean playing = false, destroyed = false;
	private short tick = -1;
	private short volume = 100;
	
	private Thread playerThread;
	
	public MauPlayer(MauSong song) {
		this.song = song;
	}
	
	public MauSong getSong() {
		return song;
	}
	
	public void play() {
		createThread();
		this.playing = true;
	}
	
	public void stop() {
		synchronized (this) {
			destroyed = true;
			playing = false;
			this.tick = (short) -1;
		}
	}
	
	public void pause() {
		this.playing = false;
	}
	
	public void unpause() {
		this.playing = true;
	}
	
	public short getVolume() {
		return volume;
	}
	
	public void setVolume(short volume) {
		this.volume = volume;
	}
	
	public void createThread() {
		playerThread = new Thread(new Runnable() {
			@Override
			public void run() {
				while (!destroyed) {
					long startTime = System.currentTimeMillis();
					synchronized (this) {
						if (playing) {
							tick++;
							if (tick > song.getLength()) {
								playing = false;
								tick = -1;
								stop();
								return;
							}
							if (!playTick(tick)) stop();
							
						}
					}
					long duration = System.currentTimeMillis() - startTime;
					float delayMillis = song.getDelay() * 50;
					if (duration < delayMillis) {
						try {
							Thread.sleep((long) (delayMillis - duration));
						} catch (InterruptedException e) {}
					}
				}
			}
		});
		playerThread.setPriority(Thread.MAX_PRIORITY);
		playerThread.start();
	}
	
	public abstract boolean playTick(int tick);
}
