package net.maunium.bukkit.MauSongPlayer;

import java.io.File;
import java.io.IOException;

import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.metadata.FixedMetadataValue;
import org.bukkit.metadata.MetadataValue;
import org.bukkit.plugin.java.JavaPlugin;

import net.maunium.bukkit.MauBukLib.I18n;
import net.maunium.bukkit.MauBukLib.I18n.Internationalizer;

public class MauSongPlayer extends JavaPlugin implements Internationalizer, Listener {
	
	public String version;
	public final String name = "MauSongPlayer", author = "Tulir293", stag = ChatColor.DARK_GREEN + "[" + ChatColor.GREEN + name + ChatColor.DARK_GREEN + "] " + ChatColor.GRAY,
			errtag = ChatColor.DARK_RED + "[" + ChatColor.RED + name + ChatColor.DARK_RED + "] " + ChatColor.RED, metatag = "MauCurrentlyPlayingSong";
	private I18n i18n;
	
	@Override
	public void onEnable() {
		File f = new File(this.getDataFolder(), "custom.lang");
		if (!f.exists()) this.saveResource("msp.lang", true);
		else new File(this.getDataFolder(), "msp.lang").delete();
		
		try {
			i18n = new I18n(this.getDataFolder(), f.exists() ? "custom" : "msp");
		} catch (IOException e) {
			e.printStackTrace();
		}
		this.getServer().getPluginManager().registerEvents(this, this);
	}
	
	@Override
	public void onDisable() {
		for (Player p : getServer().getOnlinePlayers()) {
			if (p.hasMetadata(metatag)) {
				MetadataValue mv = p.getMetadata(metatag).get(0);
				Object o = mv.value();
				if (o instanceof MauPlayer) {
					MauPlayer mp = (MauPlayer) o;
					mp.stop();
					p.removeMetadata(metatag, this);
				}
			}
		}
	}
	
	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label, String[] args) {
		if (!(sender instanceof Player)) {
			sender.sendMessage(errtag + "You must be in-game to use the MauSongPlayer");
			return true;
		}
		Player p = (Player) sender;
		if (args.length > 0) {
			if (args[0].equalsIgnoreCase("list")) {
				StringBuffer sb = new StringBuffer();
				for (File f : this.getDataFolder().listFiles()) {
					if (f.getName().endsWith(".nbs")) {
						sb.append(ChatColor.GOLD);
						sb.append(f.getName().substring(0, f.getName().length() - 4));
						sb.append(ChatColor.GRAY);
						sb.append(", ");
					}
				}
				sb.delete(sb.length() - 4, sb.length());
				p.sendMessage(stag + format("list", sb.toString()));
			} else if (args[0].equalsIgnoreCase("pause")) {
				MauPlayer mp = getPlayer(p);
				if (mp != null) {
					mp.pause();
					p.sendMessage(stag + format("paused", mp.getSong().getTitle()));
				} else p.sendMessage(errtag + format("notplaying"));
			} else if (args[0].equalsIgnoreCase("unpause") || args[0].equalsIgnoreCase("continue")) {
				MauPlayer mp = getPlayer(p);
				if (mp != null) {
					mp.unpause();
					p.sendMessage(stag + format("unpaused", mp.getSong().getTitle()));
				} else p.sendMessage(errtag + format("notplaying"));
			} else if (args[0].equalsIgnoreCase("stop")) {
				MauPlayer mp = getPlayer(p);
				if (mp != null) {
					mp.stop();
					p.sendMessage(stag + format("stopped", mp.getSong().getTitle()));
				} else p.sendMessage(errtag + format("notplaying"));
				p.removeMetadata(metatag, this);
			} else if (args[0].equalsIgnoreCase("volume")) {
				MauPlayer mp = getPlayer(p);
				if (mp != null) {
					if (args.length > 1) {
						mp.setVolume(Short.parseShort(args[1]));
						p.sendMessage(stag + format("volume.set", mp.getVolume()));
					} else p.sendMessage(stag + format("volume", mp.getVolume()));
				} else p.sendMessage(errtag + format("notplaying"));
			} else if (args.length > 1) {
				if (args[0].equalsIgnoreCase("play")) {
					MauPlayer mp = getPlayer(p);
					if (mp != null) mp.stop();
					p.removeMetadata(metatag, this);
					
					StringBuffer sb = new StringBuffer();
					for (int i = 1; i < args.length; i++) {
						sb.append(args[i]);
						sb.append(" ");
					}
					
					sb.deleteCharAt(sb.length() - 1);
					String s = sb.toString();
					for (File f : this.getDataFolder().listFiles())
						if (f.getName().endsWith(".nbs") && f.getName().toLowerCase().startsWith(s.toLowerCase())) s = f.getName();
					
					File f = new File(this.getDataFolder(), s);
					if (f.exists()) {
						MauSong song = MauSong.read(f);
						MauPlayer player = new MauPrivatePlayer(song, p);
						p.sendMessage(stag + format("started", song.getTitle()));
						player.play();
						p.setMetadata(metatag, new FixedMetadataValue(this, player));
					} else p.sendMessage(errtag + format("notfound", s));
				} else help(p, label);
			} else help(p, label);
		} else help(p, label);
		return true;
	}
	
	public MauPlayer getPlayer(Player p) {
		if (p.hasMetadata(metatag)) {
			MetadataValue mv = p.getMetadata(metatag).get(0);
			Object o = mv.value();
			if (o instanceof MauPlayer) return (MauPlayer) o;
			else return null;
		} else return null;
	}
	
	private void help(Player p, String label) {
		p.sendMessage(stag + format("help.title"));
		p.sendMessage(format("help.play", label));
		p.sendMessage(format("help.volume", label));
		p.sendMessage(format("help.list", label));
		p.sendMessage(format("help.stop", label));
		p.sendMessage(format("help.pause", label));
		p.sendMessage(format("help.unpause", label));
	}
	
	@EventHandler
	public void onPlayerQuit(PlayerQuitEvent evt) {
		if (evt.getPlayer().hasMetadata(metatag)) {
			MetadataValue mv = evt.getPlayer().getMetadata(metatag).get(0);
			Object o = mv.value();
			if (o instanceof MauPlayer) {
				MauPlayer mp = (MauPlayer) o;
				mp.stop();
			}
			evt.getPlayer().removeMetadata(metatag, this);
		}
	}
	
	@Override
	public String format(String node, Object... replace) {
		return i18n.format(node, replace);
	}
}