package net.maunium.bukkit.MauTeams;

import java.io.Serializable;
import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.configuration.serialization.ConfigurationSerializable;
import org.bukkit.configuration.serialization.SerializableAs;
import org.bukkit.entity.Player;

import net.maunium.bukkit.MauBukLib.MauUtils;

@SerializableAs("MauTeam")
public class Team implements Serializable, ConfigurationSerializable {
	private static final long serialVersionUID = 293120001;
	
	private UUID owner;
	private Set<UUID> managers;
	private Set<UUID> members;
	private Location hq, rally;
	private String name, password;
	private boolean friendlyFire;
	private static MauTeams plugin;
	
	public static void setPlugin(MauTeams p) {
		plugin = p;
	}
	
	public Team(String name, UUID owner) {
		this.name = name;
		this.owner = owner;
		this.managers = new HashSet<UUID>();
		this.members = new HashSet<UUID>();
		this.password = null;
		this.friendlyFire = false;
		this.hq = null;
		this.rally = null;
	}
	
	public boolean isPartOfTeam(UUID uuid) {
		if (isOwner(uuid)) return true;
		else if (isManager(uuid)) return true;
		else if (isMember(uuid)) return true;
		else return false;
	}
	
	public void sendChatMessage(Player p, String message) {
		String msg = plugin.getConfig().getString("teamchat-format").replace("&", "§").replace("§§", "&&").replace("{NAME}", p.getName()).replace("{DISPNAME}", p.getDisplayName())
				.replace("{TEAM}", this.getName()).replace("{MESSAGE}", message);
		for (UUID u : managers) {
			Player x = plugin.getServer().getPlayer(u);
			if (x != null) x.sendMessage(msg);
		}
		for (UUID u : members) {
			Player x = plugin.getServer().getPlayer(u);
			if (x != null) x.sendMessage(msg);
		}
		Player x = plugin.getServer().getPlayer(owner);
		if (x != null) x.sendMessage(msg);
		plugin.getLogger().info("<" + this.getName() + " " + p.getName() + "> " + message);
	}
	
	public void broadcast(String msg) {
		for (UUID u : managers) {
			Player x = plugin.getServer().getPlayer(u);
			if (x != null) x.sendMessage(msg);
		}
		for (UUID u : members) {
			Player x = plugin.getServer().getPlayer(u);
			if (x != null) x.sendMessage(msg);
		}
		Player x = plugin.getServer().getPlayer(owner);
		if (x != null) x.sendMessage(msg);
		plugin.getLogger().info("[T-BC] " + this.getName() + ": " + msg);
	}
	
	public void setOwner(UUID newowner){
		if(managers.contains(newowner)){
			managers.remove(newowner);
			managers.add(this.owner);
			this.owner = newowner;
		}
	}
	
	public boolean hasPassword() {
		return password != null;
	}
	
	public boolean isCorrectPassword(String password) {
		try {
			MessageDigest digest = MessageDigest.getInstance("SHA-256");
			byte[] hash = digest.digest(password.getBytes("UTF-8"));
			return this.password.equals(new String(hash, "UTF-8"));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}
	
	public void setPassword(String password){
		try {
			MessageDigest digest = MessageDigest.getInstance("SHA-256");
			byte[] hash = digest.digest(password.getBytes("UTF-8"));
			this.password = new String(hash, "UTF-8");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public boolean isMember(UUID uuid) {
		return members.contains(uuid);
	}
	
	public boolean isManager(UUID uuid) {
		return managers.contains(uuid);
	}
	
	public boolean isOwner(UUID uuid) {
		return owner.equals(uuid);
	}
	
	public int add(Player p) {
		if (plugin.isInTeam(p)) return 1;
		else if (!isPartOfTeam(p.getUniqueId())) {
			members.add(p.getUniqueId());
			plugin.addToTeam(p, this);
			return 0;
		} else return 2;
	}
	
	public boolean isFriendlyFireOn() {
		return friendlyFire;
	}
	
	public void setFriendlyFire(boolean on) {
		friendlyFire = on;
	}
	
	public int promote(UUID uuid) {
		if (isMember(uuid)) {
			managers.add(uuid);
			members.remove(uuid);
			return 0;
		} else if (isOwner(uuid) || isManager(uuid)) return 1;
		else return 2;
	}
	
	public int demote(UUID uuid) {
		if (isManager(uuid)) {
			managers.remove(uuid);
			members.add(uuid);
			return 0;
		} else if (isOwner(uuid) || isMember(uuid)) return 1;
		else return 2;
	}
	
	public int remove(Player p) {
		if(isOwner(p.getUniqueId())) return 1;
		if (isMember(p.getUniqueId())) members.remove(p.getUniqueId());
		else if (isManager(p.getUniqueId())) managers.remove(p.getUniqueId());
		plugin.removeFromTeam(p);
		return 0;
	}
	
	public int remove(UUID uuid) {
		if(isOwner(uuid)) return 1;
		if (isMember(uuid)) members.remove(uuid);
		else if (isManager(uuid)) managers.remove(uuid);
		Player p = Bukkit.getPlayer(uuid);
		if (p != null) plugin.removeFromTeam(p);
		return 0;
	}
	
	public String getName() {
		return name;
	}
	
	public void setHQ(Location hq) {
		this.hq = hq;
	}
	
	public Location getHQ() {
		return hq;
	}
	
	public void setRally(Location rally) {
		this.rally = rally;
	}
	
	public Location getRally() {
		return rally;
	}
	
	public Set<UUID> getManagers() {
		return managers;
	}
	
	public Set<UUID> getMembers() {
		return members;
	}
	
	public UUID getOwner() {
		return owner;
	}
	
	@Override
	public Map<String, Object> serialize() {
		Map<String, Object> rtrn = new HashMap<String, Object>();
		rtrn.put("name", name);
		if (password != null) rtrn.put("password", password);
		else rtrn.put("password", "§notset§");
		rtrn.put("owner", owner.toString());
		List<String> managers = new ArrayList<String>();
		for (UUID u : this.managers)
			managers.add(u.toString());
		rtrn.put("managers", managers);
		List<String> members = new ArrayList<String>();
		for (UUID u : this.members)
			members.add(u.toString());
		rtrn.put("members", members);
		
		if (hq != null) rtrn.put("headquaters", MauUtils.toString(hq));
		else rtrn.put("headquaters", "§notset§");
		if (rally != null) rtrn.put("rallypoint", MauUtils.toString(rally));
		else rtrn.put("rallypoint", "§notset§");
		rtrn.put("friendlyfire", friendlyFire);
		return rtrn;
	}
	
	public Team(Map<String, Object> serialized) {
		name = serialized.get("name").toString();
		Object own = serialized.get("owner");
		if (own != null) owner = UUID.fromString(own.toString());
		this.managers = new HashSet<UUID>();
		Object man = serialized.get("managers");
		if (man != null && man instanceof List<?>) {
			List<?> mans = (List<?>) man;
			for (Object o : mans)
				managers.add(UUID.fromString(o.toString()));
		}
		this.members = new HashSet<UUID>();
		Object mem = serialized.get("members");
		if (mem != null && mem instanceof List<?>) {
			List<?> memb = (List<?>) mem;
			for (Object o : memb)
				members.add(UUID.fromString(o.toString()));
		}
		Object hqp = serialized.get("headquaters");
		if (hqp != null && !hqp.toString().equals("§notset§")) {
			try {
				hq = MauUtils.parseLocation(hqp.toString());
			} catch (Throwable t) {
				t.printStackTrace();
			}
		}
		Object rp = serialized.get("rallypoint");
		if (rp != null && !rp.toString().equals("§notset§")) {
			try {
				rally = MauUtils.parseLocation(rp.toString());
			} catch (Throwable t) {
				t.printStackTrace();
			}
		}
		Object pass = serialized.get("password");
		if (pass != null && !pass.toString().equals("§notset§")) this.password = pass.toString();
		
		friendlyFire = Boolean.parseBoolean("" + serialized.get("friendlyfire"));
	}
	
	public static Team deserialize(Map<String, Object> serialized) {
		return new Team(serialized);
	}
	
	public static Team valueOf(Map<String, Object> serialized) {
		return new Team(serialized);
	}
}
