package net.maunium.bukkit.Mauconomy;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.bukkit.OfflinePlayer;

import net.milkbowl.vault.economy.Economy;
import net.milkbowl.vault.economy.EconomyResponse;
import net.milkbowl.vault.economy.EconomyResponse.ResponseType;

public class VaultEconBridge implements Economy {
	@Override
	public boolean isEnabled() {
		return true;
	}
	
	@Override
	public String getName() {
		return "Mauconomy";
	}
	
	@Override
	public boolean hasBankSupport() {
		return false;
	}
	
	@Override
	public int fractionalDigits() {
		return 2;
	}
	
	@Override
	public String format(double amount) {
		return EconomyData.getInstance().format(amount);
	}
	
	@Override
	public String currencyNamePlural() {
		return "Maudollars";
	}
	
	@Override
	public String currencyNameSingular() {
		return "Maudollar";
	}
	
	@Override
	public boolean hasAccount(OfflinePlayer player) {
		return EconomyData.getInstance().getBalance(player.getUniqueId()) == 0;
	}
	
	@Override
	public double getBalance(OfflinePlayer player) {
		return EconomyData.getInstance().getBalance(player.getUniqueId());
	}
	
	@Override
	public boolean has(OfflinePlayer player, double amount) {
		return getBalance(player) >= amount;
	}
	
	@Override
	public EconomyResponse withdrawPlayer(OfflinePlayer player, double amount) {
		EconomyData ed = EconomyData.getInstance();
		double balance = 0;
		balance = ed.getBalance(player.getUniqueId());
		
		if (balance - amount < 0) return new EconomyResponse(0, balance, ResponseType.FAILURE, "Not enough money");
		try {
			ed.setBalance(player.getUniqueId(), balance - amount);
		} catch (SQLException e) {
			return new EconomyResponse(0, balance, ResponseType.FAILURE, e.getMessage());
		}
		
		return new EconomyResponse(amount, getBalance(player), ResponseType.SUCCESS, null);
	}
	
	@Override
	public EconomyResponse depositPlayer(OfflinePlayer player, double amount) {
		EconomyData ed = EconomyData.getInstance();
		double balance = 0;
		balance = ed.getBalance(player.getUniqueId());
		
		try {
			ed.setBalance(player.getUniqueId(), balance + amount);
		} catch (SQLException e) {
			return new EconomyResponse(0, balance, ResponseType.FAILURE, e.getMessage());
		}
		
		return new EconomyResponse(amount, getBalance(player), ResponseType.SUCCESS, null);
	}
	
	@Override
	public boolean createPlayerAccount(OfflinePlayer player) {
		return EconomyData.getInstance().createAccount(player.getUniqueId());
	}
	
	/*
	 * World-based stuff
	 */
	
	@Override
	public boolean createPlayerAccount(OfflinePlayer player, String worldName) {
		return createPlayerAccount(player);
	}
	
	@Override
	public EconomyResponse depositPlayer(OfflinePlayer player, String worldName, double amount) {
		return depositPlayer(player, amount);
	}
	
	@Override
	public boolean hasAccount(OfflinePlayer player, String worldName) {
		return hasAccount(player);
	}
	
	@Override
	public double getBalance(OfflinePlayer player, String world) {
		return getBalance(player);
	}
	
	@Override
	public boolean has(OfflinePlayer player, String worldName, double amount) {
		return has(player, amount);
	}
	
	@Override
	public EconomyResponse withdrawPlayer(OfflinePlayer player, String worldName, double amount) {
		return withdrawPlayer(player, amount);
	}
	
	/*
	 * Bank stuff
	 */
	
	@Override
	public EconomyResponse createBank(String name, OfflinePlayer player) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	public EconomyResponse deleteBank(String name) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	public EconomyResponse bankBalance(String name) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	public EconomyResponse bankHas(String name, double amount) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	public EconomyResponse bankWithdraw(String name, double amount) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	public EconomyResponse bankDeposit(String name, double amount) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	public EconomyResponse isBankOwner(String name, OfflinePlayer player) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	public EconomyResponse isBankMember(String name, OfflinePlayer player) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	public List<String> getBanks() {
		return new ArrayList<String>();
	}
	
	/*
	 * Deprecated stuff
	 */
	
	@Override
	@Deprecated
	public boolean has(String playerName, double amount) {
		return false;
	}
	
	@Override
	@Deprecated
	public boolean has(String playerName, String worldName, double amount) {
		return false;
	}
	
	@Override
	@Deprecated
	public double getBalance(String playerName, String world) {
		return 0;
	}
	
	@Override
	@Deprecated
	public boolean hasAccount(String playerName, String worldName) {
		return false;
	}
	
	@Override
	@Deprecated
	public boolean hasAccount(String playerName) {
		return false;
	}
	
	@Override
	@Deprecated
	public double getBalance(String playerName) {
		return 0;
	}
	
	@Override
	@Deprecated
	public EconomyResponse depositPlayer(String playerName, String worldName, double amount) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	@Deprecated
	public EconomyResponse depositPlayer(String playerName, double amount) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	@Deprecated
	public EconomyResponse createBank(String name, String player) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	@Deprecated
	public EconomyResponse isBankOwner(String name, String playerName) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	@Deprecated
	public EconomyResponse isBankMember(String name, String playerName) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	@Deprecated
	public boolean createPlayerAccount(String playerName) {
		return false;
	}
	
	@Override
	@Deprecated
	public boolean createPlayerAccount(String playerName, String worldName) {
		return false;
	}
	
	@Override
	@Deprecated
	public EconomyResponse withdrawPlayer(String playerName, double amount) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
	
	@Override
	@Deprecated
	public EconomyResponse withdrawPlayer(String playerName, String worldName, double amount) {
		return new EconomyResponse(0, 0, ResponseType.NOT_IMPLEMENTED, null);
	}
}
