package net.maunium.bukkit.MaumbatLog;

import java.util.List;

import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.entity.Arrow;
import org.bukkit.entity.Player;
import org.bukkit.entity.ThrownPotion;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.metadata.FixedMetadataValue;
import org.bukkit.metadata.MetadataValue;
import org.bukkit.plugin.java.JavaPlugin;

public class MaumbatLog extends JavaPlugin implements Listener {
	
	private String version, name, author, TAG_KEY = "maumbatlog-tag";
	private List<String> combatLoggers;
	private int tagTime = 5000;
	
	@Override
	public void onEnable() {
		long st = System.currentTimeMillis();
		version = this.getDescription().getVersion();
		name = this.getDescription().getName();
		author = this.getDescription().getAuthors().get(0);
		this.saveDefaultConfig();
		
		this.getServer().getPluginManager().registerEvents(this, this);
		tagTime = getConfig().getInt("tag-time");
		combatLoggers = getConfig().getStringList("combat-loggers");
		
		int et = (int) (System.currentTimeMillis() - st);
		getLogger().info(name + " v" + version + " by " + author + " enabled in " + et + "ms.");
	}
	
	@Override
	public void onDisable() {
		long st = System.currentTimeMillis();
		
		getConfig().set("combat-loggers", combatLoggers);
		
		int et = (int) (System.currentTimeMillis() - st);
		getLogger().info(name + " v" + version + " by " + author + " disabled in " + et + "ms.");
	}
	
	@EventHandler
	public void onPlayerDamage(EntityDamageByEntityEvent evt) {
		if (!(evt.getEntity() instanceof Player)) return;
		Player p = (Player) evt.getEntity();
		Player dmgr;
		if (evt.getDamager() instanceof Player) dmgr = (Player) evt.getDamager();
		else if (evt.getDamager() instanceof Arrow) {
			Arrow arr = (Arrow) evt.getDamager();
			if (arr.getShooter() instanceof Player) dmgr = (Player) arr.getShooter();
			else return;
		} else if (evt.getDamager() instanceof ThrownPotion) {
			ThrownPotion pot = (ThrownPotion) evt.getDamager();
			if (pot.getShooter() instanceof Player) dmgr = (Player) pot.getShooter();
			else return;
		} else return;
		
		setMetadata(p, TAG_KEY, System.currentTimeMillis() + tagTime);
		setMetadata(dmgr, TAG_KEY, System.currentTimeMillis() + tagTime);
	}
	
	@EventHandler
	public void onPlayerDeath(PlayerDeathEvent evt) {
		evt.getEntity().removeMetadata(TAG_KEY, this);
	}
	
	@EventHandler
	public void onPlayerJoin(PlayerJoinEvent evt) {
		Player p = evt.getPlayer();
		if (combatLoggers.contains(p.getName())) {
			p.getInventory().clear();
			p.getInventory().setArmorContents(new ItemStack[p.getInventory().getArmorContents().length]);
			p.setHealth(0);
			p.sendMessage(this.getConfig().getString("join-message").replace("&", "�"));
			combatLoggers.remove(p.getName());
		}
	}
	
	@EventHandler
	public void onPlayerQuit(PlayerQuitEvent evt) {
		MetadataValue mv = getMetadata(evt.getPlayer(), TAG_KEY);
		if (mv != null) if (mv.asLong() > System.currentTimeMillis()) {
			Player p = evt.getPlayer();
			Location l = p.getLocation();
			for (ItemStack is : p.getInventory().getContents())
				if (is != null) if(!is.getType().equals(Material.AIR)) evt.getPlayer().getWorld().dropItemNaturally(l, is);
			for (ItemStack is : p.getInventory().getArmorContents())
				if (is != null) if(!is.getType().equals(Material.AIR)) evt.getPlayer().getWorld().dropItemNaturally(l, is);
			combatLoggers.add(p.getName());
			getServer().broadcastMessage(this.getConfig().getString("combatlog-broadcast").replace("&", "�"));
		}
	}
	
	public void setMetadata(Player p, String key, Object value) {
		p.setMetadata(key, new FixedMetadataValue(this, value));
	}
	
	public boolean equals(MaumbatLog m) {
		return true;
	}
	
	public MetadataValue getMetadata(Player p, String key) {
		List<MetadataValue> mvs = p.getMetadata(key);
		for (MetadataValue mv : mvs)
			if (mv.getOwningPlugin().equals(this)) return mv;
		return null;
	}
}
