package net.maunium.bukkit.PlayerLauncher;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class ActionLogger {
	private BufferedWriter logWriter;
	private SimpleDateFormat sdf;
	private boolean writing;
	public ActionLogger(File logDir) throws IOException{
		logWriter = new BufferedWriter(new FileWriter(logDir));
		sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
	}
	public boolean flush() throws IOException{
		if(writing) return false;
		logWriter.flush();
		return true;
	}
	private void log(String text, String type, String date){
		writing = true;
		try {
			// FORMAT: [Type] @ [Date] | Message
			// Example: Warning @ 23.05.2014 12:34:56 | Someone did something bad.
			logWriter.write("[" + type + "] @ [" + date + "] |" + text);
		} catch (IOException e) {
			writing = false;
			e.printStackTrace();
		}
		writing = false;
	}
	public void log(Object text, LogType type){
		String d = sdf.format(new Date());
		String t = LogType.toString(type);
		this.log(text.toString(), t, d);
	}
	public void info(Object text){
		String d = sdf.format(new Date());
		this.log(text.toString(), LogType.toString(LogType.INFO), d);
	}
	public void warning(Object text){
		String d = sdf.format(new Date());
		this.log(text.toString(), LogType.toString(LogType.WARNING), d);
	}
	public void error(Object text){
		String d = sdf.format(new Date());
		this.log(text.toString(), LogType.toString(LogType.ERROR), d);
	}
	public void severe(Object text){
		String d = sdf.format(new Date());
		this.log(text.toString(), LogType.toString(LogType.SEVERE), d);
	}
}
