package net.maunium.bukkit.PlayerLauncher;

import java.io.File;
import java.util.HashSet;

import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.metadata.FixedMetadataValue;

public class ConfigManager {
	PlayerLauncher plugin;
	int requiredConfVer = 1;
	public ConfigManager(PlayerLauncher plugin){
		this.plugin = plugin;
	}
	public void loadConfig(){
		plugin.debugln("Loading config...");
		plugin.launchers = new HashSet<Location>();
		plugin.debugln("Checking version...");
		if(plugin.getConfig().getInt("config-version") != requiredConfVer){
			loadOldConfig();
			return;
		}
		
		plugin.debugln("Loop starting up...");
		for(int i = 1; i < plugin.getConfig().getInt("amount-of-launchers") + 1; i++){
			plugin.debug("Loop " + i + " - ");
			String world = plugin.getConfig().getString("launchers." + i + ".world");
			plugin.debug("World " + world);
			World w = null;
			if(world != null){
				if(plugin.getServer().getWorld(world) != null){
					plugin.debug(" exists - ");
					w = plugin.getServer().getWorld(world);
				} else {
					plugin.debug(" does not exist!!");
					System.err.println("Configuration error when parsing " + i + "'s world (getWorld null)! Resetting...");
					brokenConfig();
				}
			} else {
				plugin.debug(" does not exist!!");
				System.err.println("Configuration error when parsing " + i + "'s world (string world null)! Resetting...");
				brokenConfig();
			}
			String blockLoc = plugin.getConfig().getString("launchers." + i + ".block-loc");
			plugin.debug("block location " + blockLoc + " - ");
			String[] blTemp = blockLoc.split(", ");
			Block b = null;
			try{
				b = w.getBlockAt(
						Integer.parseInt(blTemp[0]),
						Integer.parseInt(blTemp[1]),
						Integer.parseInt(blTemp[2]));
			} catch (NumberFormatException e){
				System.err.println("Configuration error when parsing " + i + "'s block location! Resetting...");
				brokenConfig();
			}
			
			plugin.debug("adding to list... ");
			plugin.launchers.add(b.getLocation());
			plugin.debug("Added. setting metadata... ");
			b.setMetadata("isLauncher", new FixedMetadataValue(plugin, true));
			
			plugin.debug("isLauncher set, setting vector ");
			String vector = plugin.getConfig().getString("launchers." + i + ".vector");
			plugin.debug(vector);
			String[] vTemp = vector.split(", ");
			try{
				plugin.setBlockMetadata("launcher-x", Double.parseDouble(vTemp[0]), b);
				plugin.setBlockMetadata("launcher-y", Double.parseDouble(vTemp[1]), b);
				plugin.setBlockMetadata("launcher-z", Double.parseDouble(vTemp[2]), b);
			} catch (NumberFormatException e){
				b.removeMetadata("isLauncher", plugin);
				System.err.println("Configuration error when parsing " + i + "'s launch vector! Resetting...");
				brokenConfig();
			}
			plugin.debug(". All is Guude\n");
		}
		plugin.debugln("Out of loop");
	}
	public void saveConfig(){
		File cfgBak = new File(plugin.getDataFolder(), "config.yml.bak");
		if(cfgBak.exists()){
			cfgBak.delete();
		}
		File config = new File(plugin.getDataFolder(), "config.yml");
		config.renameTo(cfgBak);
		plugin.saveDefaultConfig();
		plugin.reloadConfig();
		
		int i = 0;
		for(Location l : plugin.launchers){
			i++;
			int x, y, z;
			double vx, vy, vz;
			World w = l.getWorld();
			x = l.getBlockX();
			y = l.getBlockY();
			z = l.getBlockZ();
			
			Block b = l.getBlock();
			vx = plugin.getBlockMetadata("launcher-x", b);
			vy = plugin.getBlockMetadata("launcher-y", b);
			vz = plugin.getBlockMetadata("launcher-z", b);
			
			String blockLoc = x + ", " + y + ", " + z;
			plugin.getConfig().set("launchers." + i + ".block-loc", blockLoc);
			
			String vector = vx + ", " + vy + ", " + vz;
			plugin.getConfig().set("launchers." + i + ".vector", vector);
			
			plugin.getConfig().set("launchers." + i + ".world", w.getName());
		}
		
		plugin.getConfig().set("config-version", requiredConfVer);
		plugin.getConfig().set("amount-of-launchers", i);
		plugin.saveConfig();
	}
	public void loadOldConfig(){
		
		if(plugin.getConfig().getInt("config-version") == 0){
			plugin.getLogger().warning("Outdated configuration! Using config loader v0...");
			plugin.getLogger().warning("PS. Don't worry about that. When I save the config, I will update the format.");
			for(int i = 1; i < plugin.getConfig().getInt("amount-of-launchers") + 1; i++){
				World w = plugin.getServer().getWorld(plugin.getConfig().getString("launchers." + i + ".world"));
				Block b = w.getBlockAt(
						plugin.getConfig().getInt("launchers." + i + ".block-x"),
						plugin.getConfig().getInt("launchers." + i + ".block-y"),
						plugin.getConfig().getInt("launchers." + i + ".block-z"));
				plugin.launchers.add(b.getLocation());
				b.setMetadata("isLauncher", new FixedMetadataValue(plugin, true));
				plugin.setBlockMetadata("launcher-x", plugin.getConfig().getDouble("launchers." + i + ".launcher-x"), b);
				plugin.setBlockMetadata("launcher-y", plugin.getConfig().getDouble("launchers." + i + ".launcher-y"), b);
				plugin.setBlockMetadata("launcher-z", plugin.getConfig().getDouble("launchers." + i + ".launcher-z"), b);
			}
		}
	}
	public void brokenConfig(){
		File f = new File(plugin.getDataFolder(), "config.yml");
		f.renameTo(new File(plugin.getDataFolder(), "config.yml.broken-" + (System.currentTimeMillis() * 1000)));
		plugin.saveDefaultConfig();
		System.err.println("|-------------------------------------------------------------------|");
		System.err.println("|                                                                   |");
		System.err.println("|       Please reload the server to reactivate PlayerLauncher       |");
		System.err.println("|   Requiring a reload was caused by the corruption of the config   |");
		System.err.println("|       The broken config was not removed, but it was renamed       |");
		System.err.println("|                                                                   |");
		System.err.println("|-------------------------------------------------------------------|");
		plugin.getPluginLoader().disablePlugin(plugin);
	}
}